
; BIOS fuer ZMEIC

		TITLE	MEICBIOS

		.Z80


; SUB_AUTO        EQU     TRUE
SUB_AUTO        EQU     FALSE

; **********************************************************
;                   BIOS Sprungvektoren
; **********************************************************




             ; IM FOLGENDEN STEHEN DIE 17 VON CPM DEFINIERTEN
             ; SPRUNGVEKTOREN ZU DEN EINZELNEN EIN-/AUSGABE-
             ; FUNKTIONEN UND DISKETTEN-ROUTINEN

		JP	CBOOT		; (00) Kaltstartroutine
WBOOTE:		JP	WBOOT		; (01) Warmstartroutine
		JP	CONST		; (02) CONSOLE-Status abfragen
		JP	CONIN		; (03) CONSOLE-Eingabe
		JP	CONOUT		; (04) CONSOLE-Ausgabe
		JP	LIST		; (05) LIST-Ausgabe
		JP	SOUND		; (06) SOUND-Ausgabe
		JP	READER		; (07) READER-Eingabe
		JP	HOME		; (08) Spur Null einstellen
		JP	SELDSK		; (09) Laufwerk auswhlen
		JP	SETTRK		; (10) Spur auswhlen
		JP	SETSEC		; (11) Sektor auswhlen
		JP	SETDMA		; (12) Datenpufferadresse setzen
		JP	READ		; (13) Selektierten Sektor lesen
		JP	WRITE		; (14) Selektierten Sektor schreiben
		JP	LISTST		; (15) LIST-Status abfragen
		JP	SECTRAN		; (16) Umrechnen Sektornumme

;		JP	Monitor		; Monitor



; **********************************************************
;                  Hardware Speicherzellen
; **********************************************************


SECTOR:		DB	00H		; Sektor
TRACK:		DW	0002H		; Spur
DISKNO:		DB	00H		; Laufwerksnummer
DMAAD:		DW	80H		; DMA-Adresse


ANZ_LW		EQU	5		; Anzahl der Laufwerke (von 1 beginnend)
; Laufwerk A: -> lokale RAM-Floppy oberhalb des 64K Berechs von CP/M
; Laufwerk B: -> Flasch-Floppy der seriellen I2C Schaltkreise (4x64KB oder 4x128KB)
; Laufwerk C: -> Laufwerk ueber die serielle Schnittstelle, entweder ueber
;                den USB-Serial-Wandler an einen PC oder ueber den RPI
; Laufwerk D: -> CP/M Laufwerk ueber ein Imagefile auf einem USB-Stick
;                angeschlossen ueber die ITP3-Schnittstelle und einem CH376 Modul


DMA_ARRAY:
		DS	128,0







; **********************************************************
;                   MONITOR Speicherzellen
; **********************************************************



ZK_START:
		DB	0CH		; Clear Screen
		DB	'CP/M 2.2 Copyright 1979 (c) by Digital Research',0DH,0AH
		DB	'BIOS-Monitor 2022 written by Ronald Daleske',0DH,0AH,0DH,0AH,0

ZK_AUTO:
		DB	'SUBMIT AUTOEXEC',0




ZK_BIOS_ER01:
		DB	0DH,0AH,'BIOS Fehler - Laufwerk nicht vorhanden ...',0DH,0AH,0




; **********************************************************
;                   BIOS Speicherzellen und Texte
; **********************************************************


temp:		DB	00H




; **********************************************************
;                   BIOS Routinen
; **********************************************************






; *** CBOOT ***
; BIOS-Funktion (00) : Kaltstartroutine
; CBOOT, Cold BOOT : Kaltstart ausfhren
;
; Der Kaltstart wird nur nach dem erstmaligen Laden des Betriebsystems
; bentigt.
; Der Aufruf an den Kaltstart erfolgt meist von einem speziellen
; Ladeprogramm, das nach dem Einschalten des Rechners, das
; BIOS von der Systemdiskette geladen hat.
; Aufgabe des Kaltstart ist es, die einzelnen Systemkomponenten
; zu initialisieren und eine Meldung ber den erfolgten Systemstart
; auf der Console auszugeben.
; Falls der CCP und das BDOS bereits im Speicher stehen, kann
; nach dem Kaltstart sofort der CCP aktiviert werden. In diesem
; Fall mssen aber die beiden Sprungbefehle an den Adressen
; BOOT (nach BIOS+0003h) und BOOT+0005h (nach FBASE+OOO6h)
; eingesetzt sein. Normalerweise wird nach einem Kaltstart ein
; Warmstart ausgefhrt, der den CCP und das BDOS von der Diskette
; ldt und die Sprungbefehle einsetzt. Dann braucht nur
; die zuerst anzuwhlende Disk- und Usernummer in die Speicheradresse
; BOOT+0004h eingetragen zu werden.
;
; Aufruf: CBOOT
;
; Uebergabeparameter: keine
; Rueckgabeparameter: Sprung zum CCP oder bergang in die Warmstart-Funktion
;

CBOOT:		

		;
		LD	SP,BIOSSTACK	; STACKPOINTER SETZEN
		;






		LD	HL,ZK_START	; Anzeige des Kaltstarttextes
		CALL	AUS_ZK
		;

                ; Ausgabe der angeschlossenen Laufwerke
                ; wird vom uC an die Console gegeben
		LD	A,IORQ_LW_CONOUT
		OUT	(AUSGANG),A
                ;





; ******************************************************************

; SOUND

		LD	A,SND_START
		LD      C,A
                CALL    SOUND



; ******************************************************************





		XOR	A         	;AKKU loeschen
		LD	(IOBYTE),A	;I/O-BYTE loeschen
		LD	(USERLW),A 	;LAUFWERK 0
		;






		; *** Kopieren des BDOS in den RAM ***
		CALL	COPY_BDOS_TO_RAM



		; *** Kopieren des CCP in den RAM ***
		CALL	COPY_CCP_TO_RAM







IF SUB_AUTO

		LD	BC,ZK_AUTO	; BC = address of AUTOEXEC command string
		LD	HL,ZK_CCP	; HL = address of the first char of CCP input string

; Copy the AUTOEXEC command string into the CCP input buffer
BUFCOPY:
		LD	A,(BC)          ; A = current command string char
		CP	EOS             ; ist Zeichenkette zu Ende?
		JR	Z,BUFCOPYEND    ; ... wenn ja, dann Sprung

		LD	(HL),A          ; ... wenn nicht, dann lade das Zeichen
                                        ; in A in den CCP-Puffer
		INC     BC              ; ZK_AUTO + 1 (Quelle)
		INC     HL              ; ZK_CCP + 1 (Ziel)

		JR	BUFCOPY         ; kopiere das naechste Zeichen

; Calculate command string lenght and store it to CCP input buffer lenght variable
BUFCOPYEND:
		LD	BC,ZK_CCP	; BC = address of the first char of CCP input string
		XOR	A               ; Carry-Flag = 0
		SBC     HL, BC          ; L = Laenge der Zeichenkette (H = 0 always)
		LD      A, L            ; A = Laenge der Zeichenkette
		LD	(ZK_LEN),A      ; Abspeichern der Laenge in die
                                        ; CCP-Variable fuer die Lange der
                                        ; Zeichenkette

ENDIF


		JR	GOCPM         ;



; (01) Warmstartroutine
WBOOT:
		;
		LD	SP,BIOSSTACK	;STACKPOINTER SETZEN
		;
		;


		; *** Kopieren des BDOS in den RAM ***
		CALL	COPY_BDOS_TO_RAM



		; *** Kopieren des CCP in den RAM ***
		CALL	COPY_CCP_TO_RAM










		;Parameter setzen und Sprung zum SCCP
		;
GOCPM:		LD	A,0C3H		;JP-OPCODE
		LD	(0),A		;fuer JP BWBOOT
		LD	HL,WBOOTE	;WBOOT-Eintrittspunkt
		LD	(1),HL		;Adressfeld setzen
		;
		LD	(5),A		;fuer JP BDOS
		LD	HL,SBDOS	;SBDOS-Eintrittspunkt
		LD	(6),HL		;Adressfeld setzen
		;
		LD	BC,80H		;Standard-DMA-Adresse
		CALL	SETDMA		;setzen
		;
		LD	A,(USERLW)	;momentanes Laufwerk
		LD	C,A		;in C uebergeben
		;

IF SUB_AUTO
		JP	CCPBASE		;Sprung zum CCP
ELSE
		JP	SCCP		;Sprung zum SCCP
ENDIF



; *** CONST ***
; BIOS-Funktion (02) : CONSOLE-Status abfragen
; CONST: Status der Eingabeschnittstelle. Wurde ein Zeichen eingegeben,
; so wird im Register A der Wert OFFH bergeben, sonst 0. 
; Das Zeichen wird aber noch nicht eingelesen.
; Diese BIOS-Funktion wird vom BDOS bei jeder Zeichenausgabe
; zur Erkennung der Steuerzeichen CTRL-S und CTRL-C aufgerufen.
; Aufruf: CONST
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = 000H -> kein Zeichen
;                     Register A = 0FFH -> Zeichen vorhanden
;
CONST:
		LD	A,IORQ_CONST
		OUT	(AUSGANG),A

		IN	A,(AUSGANG)

		; ist ein Eingabewert vorhanden, so ist 
		; Register A groesser 0
		CP	0
		JR	Z,CONST_A

		LD	A,0FFH

CONST_A:
		; Ruecksprung
		;
		RET



; *** CONIN ***
; BIOS-Funktion (03) : CONSOLE-Eingabe
; CONIN: A = Zeichen von der Console
; Diese Routine holt ein Zeichen von der Console (Tastatur) und
; gibt es im A-Register zurck. Falls kein Zeichen von der
; Console ansteht, mu auf ein Zeichen gewartet werden.
; Aufruf: CONIN
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = Zeichen von der Console
;    
CONIN:
		LD	A,IORQ_CONIN
		OUT	(AUSGANG),A

		IN	A,(AUSGANG)
		;
		CP	0
		JR	Z,CONIN
		;

		; Ruecksprung
		;
		RET



; *** CONOUT ***
; BIOS-Funktion (04) : CONSOLE-Ausgabe
; CONOUT (CONsole OUTput) : Zeichen zur Console ausgeben, 
; Der im Register C befindliche Wert wird zur Console (Bildschirm) ausgegeben.
; Aufruf: CONOUT
; Uebergabeparameter: Register C = Zeichen im ASCII-Code
; Rueckgabeparameter: keine
;
  
CONOUT:
		LD	A,IORQ_CONOUT
		OUT	(AUSGANG),A

		LD	A, C
		OUT	(AUSGANG),A 

		; Ruecksprung
		;
		RET







; *** LIST ***
; BIOS-Funktion (05) : LIST-Ausgabe
; LIST (LIST output) : Zeichen zum 'List '-Kanal (Drucker) ausgeben 
; Wie Consolenausgabe, aber zum ' List ' -Kanal.
;
; Aufruf: LIST
; Uebergabeparameter: Register C = Zeichen im ASCII-Code
; Rueckgabeparameter: keine
;   
LIST:
		LD	A,IORQ_LIST
		OUT	(AUSGANG),A

		LD	A, C
		OUT	(AUSGANG),A 

		; Ruecksprung
		;
		RET





; *** SOUND ***
; BIOS-Funktion (06) : PUNCH-Ausgabe
; SOUND output : Zeichen zum ' SOUND ' -Kanal ausgeben 
; Wie Consolenausgabe, aber zum ' SOUND ' -Kanal
;
; Aufruf: PUNCH
; Uebergabeparameter: Register C = Zeichen im ASCII-Code
; Rueckgabeparameter: keine
;   
; --- wird als Sound-Ausgabe genutzt ---
SOUND:

		LD	A,IORQ_SOUND
		OUT	(AUSGANG),A

		LD	A, C
		OUT	(AUSGANG),A

		; Ruecksprung
		;
		RET






; *** READER ***
; BIOS-Funktion (07) : READER-Eingabe
; READER (READER input) : Zeichen vom ' Reader ' -Kanal holen 
; Wie Consoleneingabe , aber vom 'Reader ' -Kanal
;
; Aufruf: READER
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = Zeichen von der Console
;   
; *** derzeit noch nicht implementiert ***
READER:
		RET





; *** HOME ***
; BIOS-Funktion (08) : Spur Null einstellen
;
; Diese Funktion war bei lteren Laufwerken zur exakten
; Positionierung des Schreib/Lesekopfes gedacht.
; Da das BDOS vor jedem Diskzugriff die Spurnummer ber SETTRK
; anwhlt, ist HOHE bei neueren Laufwerken berflssig.
;
; Aufruf: HOME
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
;   
HOME:
		LD	C,0
		CALL	SETTRK
		RET




; *** SELDSK ***
; BIOS-Funktion (09)   : Laufwerk auswhlen
; SELDSK (SELect DISK) : Das BIOS mu die im C-Register bergebene 
; Laufwerksnummer berprfen und, falls ein Laufwerk mit dieser Nummer 
; existiert, in HL die Adresse des zugehrigen DPH zurckgeben,
; Im CP/n ist nicht garantiert, da nach einem SELDSK-Aufruf
; auch tatschlich auf dieses Laufwerk zugegriffen wird.
; Vielmehr hat der SELDSK-Aufruf nur eine 'Anmeldefunktion',
; damit sich das BDOS auf das Laufwerk einstellen kann.
; Das BIOS mu die Laufwerksnummer aber intern speichern, da
; sich nachfolgende Diskzugriffe immer auf das zuletzt selektierte
; Laufwerk beziehen.
;
; Aufruf: SELDSK
;
; Uebergabeparameter: C   = Laufwerksnummer
;                         =  0 fr Laufwerk A:
;                           ...
;                         = 15 fr Laufwerk P:
;
;                     E,0 = Login-Bit
;                         = 0, falls das Laufwerk zum ersten Mal
;                           angewhlt wird
;                         = 1, falls das Laufwerk seit dem letzten
;                           Warmstart schonmal angewhlt wurde
;
; Rueckgabeparameter: HL -> DPH des Laufwerks
;                     HL = 0, falls das Laufwerk nicht existiert
;  
SELDSK:
		LD	HL,0		; Code bei Fehler
		LD	A,C
		CP	ANZ_LW		; Abfrage der maximalen Laufwerksnummer
		RET	NC		; RET wenn Laufwerk nicht vorhanden
		; Laufwerksnummer im erlaubten Bereich
		LD	(DISKNO),A
		; Parameterkopf-Adresse berechnen
		LD	L,A
		LD	H,0		; HL=Laufwerksnummer
		ADD	HL,HL		; *2
		ADD	HL,HL		; *4
		ADD	HL,HL		; *8
		ADD	HL,HL		; *16 (Kopf - Laenge)
		LD 	DE,DPH
		ADD	HL,DE		; HL = DPBASE (DISKNO*16)
		;
		RET





; *** SETTRK ***
; BIOS-Funktion (10) : Spurnummer auswhlen
; SETTRK (SET TRacK) : Der nchste Diskzugriff bezieht sich auf die im 
; Registerpaar BC bergebene Spur. Die so gesetzte Spurnummer, errechnet
; sich immer aus der BDOS-internen (logischen) Spurnummer plus dem OFF-Wert im DPB. 
; Wie auch beim SELDSK-Aufruf ist ein tatschlicher Diskzugriff nicht garantiert.
;
; Aufruf: SETTRK
;
; Uebergabeparameter: Register BC = Spurnummer
; Rueckgabeparameter: keine
;  
SETTRK:
		LD	(TRACK),BC
		;
		RET




; *** SETSEC ***
; BIOS-Funktion (11)  : Sektor auswhlen
; SETSEC (SET SECtor) : Logische Sektornummer setzen 
;
; Der nchste Diskzugriff bezieht sich auf den im Registerpaar
; BC bergebenen Sektor. Die so gesetzte Sektornummer ist immer
; das Ergebnis der SECTRAN-Funktion (s.u.). Auch hier ist ein
; tatschlicher Diskzugriff auf diesen Sektor nicht garantiert.
;
; Aufruf: SETSEC
;
; Uebergabeparameter: Register BC = Logische Sektornummer - Hier wird nur das niederwertige
;                                                           Byte (Reg.C) abgespeichert 
; Rueckgabeparameter: keine
;  
SETSEC:
		LD	A,C
		LD	(SECTOR),A
		;
		RET






; *** SETDMA ***
; BIOS-Funktion (12) : Datenpufferadresse setzen
;
; SETDMA (SET DMA-addresse) : Alle nachfolgenden Diskzugriffe
; mssen die DMA-Adresse als Quell- (bei Schreibzugriffen)
; bzw. Zieladresse (bei Lesezugriffen) benutzen.
; Die DMA-Adresse zeigt immer auf einen 128-Byte groen Buffer,
; weshalb Diskzugriffe immer in Recordgre erfolgen.
;
; Aufruf: SETDMA
;
; Uebergabeparameter: Register BC = DMA-Adresse
; Rueckgabeparameter: keine
;  
SETDMA:     
	LD	(DMAAD),BC
	;
	RET




; *** READ ***
; BIOS-Funktion (13) : Selektierten Sektor lesen
; READ, READ sector  : Logischen Sektor lesen
;
; Die READ-Funktion liest einen (logischen) Sektor von der Diskette 
; in den DMA-Buffer. Die Disknummer, Spurnummer und Sektornummer sind
; jeweils durch die letzten SELDSK-, SETTRKund SETSEC-Aufrufe festgelegt.
; Bei physikalischen Sektorlaengen vom mehr als 128 Bytes, mu das BIOS einen
; Sektorbuffer entsprechender Groesse selbst bereitstellen und aus diesem Buffer,
; 128 Bytes zum zuletzt definierten DMA-Buffer kopieren.
; Falls ein Lesefehler auftritt, sollte das BIOS den Diskzugriff
; ein paar Mal wiederholen und, falls der Fehler bestehen
; bleibt, den Fehlercode 1 im A-Register zurueckgeben.
;
; Aufruf: READ
;
; Uebergabeparameter: keine
; Rueckgabeparameter: Register A = Fehlercode
;                                = 0, kein Fehler
;                                = 1, sonst
;  
READ:	

		; Laufwerk A..P (soweit vorhanden)
		; 

		; Laden von Sektor, Spur und Laufwerk an den Mikrocontroller
		CALL	STD_an_uC

                ; pruefen, ob das angesprochene Laufwerk
                ; verfuegbar ist - wenn nicht, wird
                ; eine Fehlermeldung ausgegeben
		LD	A,IORQ_LW_ok
		OUT	(AUSGANG),A
                ;
		IN	A,(AUSGANG)
                ;
                CP      0
		JR	NZ,READ2


		; Lesen eines Sektors vom PC (CONSOLE) in den Sektor_Puffer
		; des Mikrocontrollers 
		LD	A,IORQ_RD_DISK2ARR
		OUT	(AUSGANG),A

		; Kopieren des Sektors vom Sektor_Puffer des Mikrocontrollers
		; in den RAM des Z80 an die DMA-Adresse
		LD	C,0
		LD	HL,(DMAAD)
		CALL	COPY_ARR_TO_RAM


READ3:
		XOR	A		; kein Fehler
		;
		RET



READ2:
		;
		; wird ein Laufwerk angesprochen, das nicht vorhanden
		; ist (groesser B), so wird eine BIOS-Fehlermeldung ausgegeben
		;
		LD	HL,ZK_BIOS_ER01
		CALL	AUS_ZK
		;
                LD      A,0FFH
                ;
                RET







; *** WRITE ***
; BIOS-Funktion (14)  : Selektierten Sektor schreiben
; WRITE (WRITE sector): Logischen Sektor schreiben
;
; Die WRITE-Funktion schreibt einen (logischen) Sektor vom DMABuffer
; auf die Diskette. Die Disknummer, Spurnummer und Sektornummer
; sind jeweils durch die letzten SELDSK-, SETTRK- und SETSEC-Aufrufe festgelegt.
; Bei physikalischen Sektorlngen von mehr als 128 Bytes, kann das Record-Flag
; zur Realisierung eines 'Blocking'-Algorithmus verwendet werden. 
; Bei einem normalen Schreibzugriff reicht es, den logischen Sektor nur in 
; den BIOS-internen Sektorbuffer zu bernehmen.
; Dies hat den Vorteil, da nachfolgende Schreibzugriffe auf den selben
; physikalischen Sektor, keinen Diskettenzugriff verlangen.
; Erst wenn der neue logische Sektor in einem anderen physikalsichen Sektor
; liegt, mu der Sektorbuffer auf die Diskette geschrieben werden.
; Directory-Schreibzugriffe sollten immer direkt auf die
; Diskette geleitet werden.
;
; Aufruf: WRITE
;
; Uebergabeparameter: Register C = Record-Flag
;                                = 0, bei einem normalen Schreibzugriff
;                                = 1, falls der logische Sektor ein Directory-Record ist 
;                                = 2, falls der logische Sektor der erste Sektor eines neuen Blocks ist
;
; Rueckgabeparameter: Register A = Fehlercode
;                                = 0, kein Fehler
;                                = 1, sonst
;  
WRITE:
		;
		; Laufwerk A..P (soweit vorhanden)
		;


		; Kopieren des Sektors vom RAM des Z80 an der DMA-Adresse
                ; in den Sektor_Puffer des Mikrocontrollers
		;
		LD	C,0		; 0 ist der CP/M Bereich
		LD	HL,(DMAAD)
		CALL	COPY_RAM_TO_ARR


		; Laden von Sektor, Spur und Laufwerk an den Mikrocontroller
		CALL	STD_an_uC


                ; pruefen, ob das angesprochene Laufwerk
                ; verfuegbar ist - wenn nicht, wird
                ; eine Fehlermeldung ausgegeben
		LD	A,IORQ_LW_ok
		OUT	(AUSGANG),A
                ;
		IN	A,(AUSGANG)
                ;
                CP      0
		JR	NZ,WRITE2


		; Schreiben des Sektors vom Sektor_Puffer des
		; Mikrocontrollers in den I2C-Flasc
		;
		LD	A,IORQ_WR_ARR2DISK
		OUT	(AUSGANG),A


WRITE3:
		XOR	A		; kein Fehler
		;
		RET



WRITE2:
		;
		; wird ein Laufwerk angesprochen, das nicht vorhanden
		; ist (groesser B), so wird eine BIOS-Fehlermeldung ausgegeben
		;
		LD	HL,ZK_BIOS_ER01
		CALL	AUS_ZK
		;
		;
                LD      A,0FFH
                ;
                RET










; *** LISTST ***
; BIOS-Funktion (15) : LIST-Status abfragen
;
; LISTST (LIST output STatus) : Status der Druckerausgabe testen 
;
; LISTST wird vom BDOS nicht bentigt, sondern dient dem CP/M
; Hilfsprogramm 'DESPOOL1 den Status der Druckerausgabe zu
; bestimmen.
;
; Aufruf: LISTST
;
; Uebergabeparameter: keine
; Rueckgabeparameter: A = Druckerstatus
;                       = OOH, Drucker nicht bereit
;                       = FFH, Drucker frei
;  
LISTST:
		;
		LD	A,0FFH		; Drucker bereit
		;
		RET




; *** SECTRAN ***
; BIOS-Funktion (16) : Umrechnen Sektornummer
;
; SECTRAN (SECtor TRANslation) : Sektornummer aus der Sektor-Verschrnkungs-
;                                Tabelle holen 
;
; Das BDOS benutzt diese Funktion, um aus der im DPH angegebenen
; Sektor-Verschrnkungs-Tabelle die Sektornummer zu entnehmen.
; Im allgemeinen gengt es, einfach die im Registerpaar BC
; bergebene Sektornummer ins Registerpaar HL zu kopieren.
;
; Aufruf: LISTST
;
; Uebergabeparameter: Register BC = Logische Sektornummer
;                     Register DE -> Sektor-Verschrnkungs-Tabelle
;
; Rueckgabeparameter: Register HL = (Logische) Sektornummer aus der Tabelle
;  
SECTRAN:
		LD	H,B		; Laden von HL mit BC
		LD	L,C
		;
		RET




; **********************************************************





; Funktion: Ausgabe einer Zeichenkette auf den Bildschirm
;
; Aufruf: AUS_ZK
; Uebergabeparameter: Register HL = Adresse der Zeichenkette 
;                     Ende = 00H
; Rueckgabeparameter: keine
; 
AUS_ZK:		LD	A,(HL)
		OR	A
		RET	Z		;RET wenn Zeichen = 0 ist
		LD	C,A
		PUSH	HL
		CALL	CONOUT		;Ausgabe des Zeichens in Reg.C
		POP	HL
		INC	HL
		;
		JR	AUS_ZK





; Funktion: Ausgabe von CR+LF an den Bildschirm
;
; Aufruf: AUS_CRLF
; Uebergabeparameter: keine
; Rueckgabeparameter: keine
; 
AUS_CRLF:	
		LD	C,0DH
		CALL	CONOUT		;Ausgabe des Zeichens in Reg.C
		;
		LD	C,0AH
		CALL	CONOUT		;Ausgabe des Zeichens in Reg.C
		;
		RET






; Funktion: Zeitschleife von etwa 0,1 Sekunden
;
; Aufruf: WAIT_01S
; Uebergabeparameter: Register A - 1..255 in 100 mS
; Rueckgabeparameter: keine
; 
WAIT_01S_Fak	EQU	FRQ*354/10
;
WAIT_01S:	LD	C,WAIT_01S_Fak	; 7T
		;
WAIT_01S_B:	LD	B,254		; 7T
		;
WAIT_01S_C:	DEC	B		; 4T
		JR	NZ,WAIT_01S_C	; 7T / 12T / 11T*254=2794T+12T+7T=2813T
		;
		DEC	C		; 4T
		JR	NZ,WAIT_01S_B	; 7T / 12T / 2813T+11T=2824T*141=398184T+12T+7T=398203T
		;
		DEC	A
		JR	NZ,WAIT_01S

		RET





; **********************************************************
;  Datenbereich
; **********************************************************

DPH:
        DW 0000h,0000h,0000h,0000h,DIRBUF,DPB2,0000h,ALV00  ; LW A:
        DW 0000h,0000h,0000h,0000h,DIRBUF,DPB2,0000h,ALV01  ; LW B:
        DW 0000h,0000h,0000h,0000h,DIRBUF,DPB2,0000h,ALV02  ; LW C:
        DW 0000h,0000h,0000h,0000h,DIRBUF,DPB2,0000h,ALV03  ; LW D:
        DW 0000h,0000h,0000h,0000h,DIRBUF,DPB2,0000h,ALV04  ; LW E:


DPB2:
        DW 128  ;SPT - 128 bytes sectors per track
        DB 5    ;BSH - block shift factor
        DB 31   ;BLM - block mask
        DB 1    ;EXM - Extent mask
        DW 2047 ;DSM - Storage size (blocks - 1)
        DW 511  ;DRM - Number of directory entries - 1
        DB 240  ;AL0 - 1 bit set per directory block
        DB 0    ;AL1 -            "
        DW 0    ;CKS - DIR check vector size (DRM+1)/4 (0=fixed disk)
        DW 0    ;OFF - Reserved tracks



















